@@ -36,8 +36,8 @@ class LinkDialog : public MasterDialog {
     void on_searchLineEdit_textChanged(const QString &arg1);
     void on_notesListWidget_doubleClicked(const QModelIndex &index);
     void on_urlEdit_textChanged(const QString &arg1);
-    void addFileUrl();
-    void addDirectoryUrl();
+    void addFileUrl(bool relative = false);
+    void addDirectoryUrl(bool relative = false);
     void slotReplyFinished(QNetworkReply *reply);
     void downloadProgress(qint64 bytesReceived, qint64 bytesTotal);
 
@@ -49,6 +49,8 @@ class LinkDialog : public MasterDialog {
 
     void on_tabWidget_currentChanged(int index);
 
+    void on_refreshButton_clicked();
+
    private:
     Ui::LinkDialog *ui;
     int firstVisibleNoteListRow;
