@@ -196,9 +196,61 @@ static Selection::Element selectionDetails(const litehtml::element::ptr &element
                                            const QString &text,
                                            const QPoint &pos)
 {
-    // shortcut, which _might_ not really be correct
-    if (element->get_children_count() > 0)
-        return {element, -1, -1}; // everything selected
+    // If element has children, the text contains all descendant text concatenated
+    // For better precision, we should find the actual leaf child at this position
+    if (element->get_children_count() > 0) {
+        // Try to find a child leaf element that contains this position
+        const std::function<Selection::Element(litehtml::element::ptr, QPoint)> findLeaf =
+            [&findLeaf, pos](const litehtml::element::ptr &elem,
+                             const QPoint &offset) -> Selection::Element {
+            if (!elem)
+                return {};
+
+            const QRect elemPos = toQRect(elem->get_position()).translated(offset);
+
+            if (!elemPos.contains(pos))
+                return {};
+
+            // Try children first
+            for (int i = 0; i < int(elem->get_children_count()); ++i) {
+                const litehtml::element::ptr child = elem->get_child(i);
+                Selection::Element result = findLeaf(child, elemPos.topLeft());
+                if (result.element)
+                    return result;
+            }
+
+            // No children found, use this element if it has text
+            litehtml::tstring elemText;
+            elem->get_text(elemText);
+            if (!elemText.empty()) {
+                const QString textStr = QString::fromStdString(elemText);
+                const QFont &font = toQFont(elem->get_font());
+                const QFontMetrics fm(font);
+                const QPoint relativePos = pos - elemPos.topLeft();
+
+                int previous = 0;
+                for (int i = 0; i < textStr.size(); ++i) {
+                    const int width = fm.size(0, textStr.left(i + 1)).width();
+                    if ((width + previous) / 2 >= relativePos.x())
+                        return {elem, i, previous};
+                    previous = width;
+                }
+                return {elem, int(textStr.size()), previous};
+            }
+
+            return {};
+        };
+
+        const QRect elementRect = toQRect(element->get_placement());
+        Selection::Element result = findLeaf(element, elementRect.topLeft());
+        if (result.element)
+            return result;
+
+        // Fallback: return the element with index at the start
+        return {element, 0, 0};
+    }
+
+    // Element has no children - calculate precise character position
     const QFont &font = toQFont(element->get_font());
     const QFontMetrics fm(font);
     int previous = 0;
@@ -219,26 +271,54 @@ static Selection::Element deepest_child_at_point(const litehtml::document::ptr &
     if (!document)
         return {};
 
-    // the following does not find the "smallest" element, it often consists of children
-    // with individual words as text...
+    const std::function<Selection::Element(litehtml::element::ptr)> firstTextLeaf =
+        [&firstTextLeaf](const litehtml::element::ptr &element) -> Selection::Element {
+        if (!element)
+            return {};
+        if (element->get_children_count() == 0) {
+            litehtml::tstring text;
+            element->get_text(text);
+            if (!text.empty())
+                return {element, 0, 0};
+            return {};
+        }
+        for (int i = 0; i < int(element->get_children_count()); ++i) {
+            const litehtml::element::ptr child = element->get_child(i);
+            Selection::Element result = firstTextLeaf(child);
+            if (result.element)
+                return result;
+        }
+        litehtml::tstring text;
+        element->get_text(text);
+        if (!text.empty())
+            return {element, 0, 0};
+        return {};
+    };
+
+    // Find the element at this point
     const litehtml::element::ptr element = document->root()->get_element_by_point(pos.x(),
                                                                                   pos.y(),
                                                                                   viewportPos.x(),
                                                                                   viewportPos.y());
-    // ...so try to find a better match
+    // Recursively find the deepest child that contains the point and has text
     const std::function<Selection::Element(litehtml::element::ptr, QRect)> recursion =
         [&recursion, pos, mode](const litehtml::element::ptr &element,
                                 const QRect &placement) -> Selection::Element {
         if (!element)
             return {};
-        Selection::Element result;
+
+        // First, try to find a deeper child that contains the point
+        Selection::Element deepestResult;
         for (int i = 0; i < int(element->get_children_count()); ++i) {
             const litehtml::element::ptr child = element->get_child(i);
-            result = recursion(child,
-                               toQRect(child->get_position()).translated(placement.topLeft()));
-            if (result.element)
-                return result;
+            const QRect childPlacement = toQRect(child->get_position())
+                                             .translated(placement.topLeft());
+            deepestResult = recursion(child, childPlacement);
+            if (deepestResult.element)
+                return deepestResult;
         }
+
+        // No deeper child found, check if this element itself has text content
         if (placement.contains(pos)) {
             litehtml::tstring text;
             element->get_text(text);
@@ -250,9 +330,15 @@ static Selection::Element deepest_child_at_point(const litehtml::document::ptr &
                            : Selection::Element({element, -1, -1});
             }
         }
+
         return {};
     };
-    return recursion(element, element ? toQRect(element->get_placement()) : QRect());
+    Selection::Element result = recursion(element,
+                                          element ? toQRect(element->get_placement()) : QRect());
+    if (result.element)
+        return result;
+
+    return firstTextLeaf(element);
 }
 
 // CSS: 400 == normal, 700 == bold.
@@ -583,7 +669,17 @@ int DocumentContainerPrivate::pt_to_px(int pt) const
 
 int DocumentContainerPrivate::get_default_font_size() const
 {
-    return m_defaultFont.pointSize();
+    int pointSize = m_defaultFont.pointSize();
+    if (pointSize <= 0) {
+        int pixelSize = m_defaultFont.pixelSize();
+        if (pixelSize > 0 && m_paintDevice) {
+            pointSize = qRound(m_paintDevice->logicalDpiY() * pixelSize / 72.0);
+        }
+    }
+    if (pointSize <= 0) {
+        pointSize = 16;
+    }
+    return pointSize;
 }
 
 const litehtml::tchar_t *DocumentContainerPrivate::get_default_font_name() const
@@ -960,6 +1056,8 @@ std::shared_ptr<litehtml::element> DocumentContainerPrivate::create_element(
     const litehtml::string_map &attributes,
     const std::shared_ptr<litehtml::document> &doc)
 {
+    // TODO
+
     // We only handle checkbox here
     if (litehtml::t_strcasecmp("input", tag_name) != 0) {
         return {};
@@ -974,18 +1072,13 @@ std::shared_ptr<litehtml::element> DocumentContainerPrivate::create_element(
 
     auto checkBox = std::make_shared<checkbox>(m_document);
     checkBox->set_checked(isChecked);
-    checkBox->set_index(m_checkboxIndex);
-
-    // Set up the click callback to emit the checkbox index
-    if (m_checkboxCallback) {
-        checkBox->set_click_callback(m_checkboxCallback);
-    }
-
-    m_checkboxIndex++;
     return checkBox;
 
+    qDebug(log) << "create_element" << QString::fromUtf8(tag_name);
+
     Q_UNUSED(attributes)
     Q_UNUSED(doc)
+    return {};
 }
 
 void DocumentContainerPrivate::get_media_features(litehtml::media_features &media) const
@@ -1018,7 +1111,6 @@ void DocumentContainer::setDocument(const QByteArray &data, DocumentContainerCon
 {
     d->m_pixmaps.clear();
     d->clearSelection();
-    d->m_checkboxIndex = 0; // Reset checkbox index for new document
     d->m_document = litehtml::document::createFromUTF8(data.constData(),
                                                        d.get(),
                                                        &context->d->context);
@@ -1077,20 +1169,32 @@ int DocumentContainer::anchorY(const QString &anchorName) const
 
 QVector<QRect> DocumentContainer::mousePressEvent(const QPoint &documentPos,
                                                   const QPoint &viewportPos,
-                                                  Qt::MouseButton button)
+                                                  Qt::MouseButton button,
+                                                  Qt::KeyboardModifiers modifiers)
 {
     if (!d->m_document || button != Qt::LeftButton)
         return {};
     QVector<QRect> redrawRects;
     // selection
-    if (d->m_selection.isValid())
-        redrawRects.append(d->m_selection.boundingRect());
-    d->clearSelection();
-    d->m_selection.selectionStartDocumentPos = documentPos;
-    d->m_selection.startElem = deepest_child_at_point(d->m_document,
-                                                      documentPos,
-                                                      viewportPos,
-                                                      d->m_selection.mode);
+    if (modifiers.testFlag(Qt::ShiftModifier) && d->m_selection.isValid()) {
+        d->m_selection.selectionStartDocumentPos = documentPos;
+        d->m_selection.endElem = deepest_child_at_point(d->m_document,
+                                                        documentPos,
+                                                        viewportPos,
+                                                        d->m_selection.mode);
+        d->updateSelection();
+        if (d->m_selection.isValid())
+            redrawRects.append(d->m_selection.boundingRect());
+    } else {
+        if (d->m_selection.isValid())
+            redrawRects.append(d->m_selection.boundingRect());
+        d->clearSelection();
+        d->m_selection.selectionStartDocumentPos = documentPos;
+        d->m_selection.startElem = deepest_child_at_point(d->m_document,
+                                                          documentPos,
+                                                          viewportPos,
+                                                          d->m_selection.mode);
+    }
     // post to litehtml
     litehtml::position::vector redrawBoxes;
     if (d->m_document->on_lbutton_down(documentPos.x(),
@@ -1238,6 +1342,229 @@ QString DocumentContainer::selectedText() const
     return d->m_selection.text;
 }
 
+// Helper function to get computed style properties from a litehtml element
+static QString getElementStyles(const litehtml::element::ptr &element)
+{
+    if (!element)
+        return QString();
+
+    QStringList styles;
+
+    // Get font family
+    const char *fontFamily = element->get_style_property("font-family", true, nullptr);
+    if (fontFamily && strlen(fontFamily) > 0) {
+        styles << QStringLiteral("font-family: %1").arg(QString::fromUtf8(fontFamily));
+    }
+
+    // Get font size
+    int fontSize = element->get_font_size();
+    if (fontSize > 0) {
+        styles << QStringLiteral("font-size: %1px").arg(fontSize);
+    }
+
+    // Get font weight
+    const char *fontWeight = element->get_style_property("font-weight", true, nullptr);
+    if (fontWeight && strlen(fontWeight) > 0 && strcmp(fontWeight, "normal") != 0) {
+        styles << QStringLiteral("font-weight: %1").arg(QString::fromUtf8(fontWeight));
+    }
+
+    // Get font style
+    const char *fontStyle = element->get_style_property("font-style", true, nullptr);
+    if (fontStyle && strlen(fontStyle) > 0 && strcmp(fontStyle, "normal") != 0) {
+        styles << QStringLiteral("font-style: %1").arg(QString::fromUtf8(fontStyle));
+    }
+
+    // Get text decoration
+    const char *textDecoration = element->get_style_property("text-decoration", true, nullptr);
+    if (textDecoration && strlen(textDecoration) > 0 && strcmp(textDecoration, "none") != 0) {
+        styles << QStringLiteral("text-decoration: %1").arg(QString::fromUtf8(textDecoration));
+    }
+
+    // Get color
+    litehtml::web_color color = element->get_color("color", true, litehtml::web_color());
+    if (color.alpha > 0) {
+        styles << QStringLiteral("color: rgb(%1, %2, %3)")
+                      .arg(color.red)
+                      .arg(color.green)
+                      .arg(color.blue);
+    }
+
+    // Get background color
+    const char *bgColor = element->get_style_property("background-color", false, nullptr);
+    if (bgColor && strlen(bgColor) > 0 && strcmp(bgColor, "transparent") != 0) {
+        litehtml::web_color bg = litehtml::web_color::from_string(bgColor, nullptr);
+        if (bg.alpha > 0) {
+            styles << QStringLiteral("background-color: rgb(%1, %2, %3)")
+                          .arg(bg.red)
+                          .arg(bg.green)
+                          .arg(bg.blue);
+        }
+    }
+
+    // Get text alignment
+    const char *textAlign = element->get_style_property("text-align", true, nullptr);
+    if (textAlign && strlen(textAlign) > 0 && strcmp(textAlign, "left") != 0) {
+        styles << QStringLiteral("text-align: %1").arg(QString::fromUtf8(textAlign));
+    }
+
+    return styles.join(QStringLiteral("; "));
+}
+
+// Helper structure to track parent elements during traversal
+struct ElementContext
+{
+    litehtml::element::ptr element;
+    QString tagName;
+    QString styles;
+    bool opened = false;
+};
+
+// Helper function to serialize a single leaf element with its parent hierarchy
+static void serializeLeafWithParents(const litehtml::element::ptr &leafElement,
+                                     QVector<ElementContext> &parentStack,
+                                     QString &html)
+{
+    if (!leafElement)
+        return;
+
+    // Build the path from root to this leaf
+    litehtml::elements_vector leafPath = path(leafElement);
+
+    // Find common ancestor with current stack
+    int commonDepth = 0;
+    const int minSize = std::min(static_cast<int>(parentStack.size()),
+                                 static_cast<int>(leafPath.size()));
+    for (int i = 0; i < minSize; ++i) {
+        if (i >= parentStack.size() || parentStack[i].element != leafPath[i]) {
+            break;
+        }
+        commonDepth = i + 1;
+    }
+
+    // Close tags that are no longer in the path
+    for (int i = parentStack.size() - 1; i >= commonDepth; --i) {
+        if (parentStack[i].opened && !parentStack[i].tagName.isEmpty()) {
+            html += QStringLiteral("</") + parentStack[i].tagName + QStringLiteral(">");
+        }
+    }
+    parentStack.resize(commonDepth);
+
+    // Open new parent tags
+    for (size_t i = commonDepth; i < leafPath.size(); ++i) {
+        const litehtml::element::ptr &elem = leafPath[i];
+        const char *tagName = elem->get_tagName();
+
+        ElementContext ctx;
+        ctx.element = elem;
+
+        if (tagName && strlen(tagName) > 0) {
+            QString tag = QString::fromUtf8(tagName);
+            ctx.tagName = tag;
+
+            // Skip html, head, body wrappers
+            if (tag == QLatin1String("html") || tag == QLatin1String("head")
+                || tag == QLatin1String("body")) {
+                ctx.opened = false;
+                parentStack.append(ctx);
+                continue;
+            }
+
+            // Open tag
+            html += QStringLiteral("<") + tag;
+
+            // Get and add inline styles
+            QString inlineStyles = getElementStyles(elem);
+            if (!inlineStyles.isEmpty()) {
+                html += QStringLiteral(" style=\"%1\"").arg(inlineStyles);
+            }
+
+            // Add certain important attributes
+            if (tag == QLatin1String("a")) {
+                const char *href = elem->get_attr("href");
+                if (href && strlen(href) > 0) {
+                    html += QStringLiteral(" href=\"%1\"").arg(QString::fromUtf8(href));
+                }
+            } else if (tag == QLatin1String("img")) {
+                const char *src = elem->get_attr("src");
+                if (src && strlen(src) > 0) {
+                    html += QStringLiteral(" src=\"%1\"").arg(QString::fromUtf8(src));
+                }
+                const char *alt = elem->get_attr("alt");
+                if (alt && strlen(alt) > 0) {
+                    html += QStringLiteral(" alt=\"%1\"").arg(QString::fromUtf8(alt));
+                }
+            }
+
+            html += QStringLiteral(">");
+            ctx.opened = true;
+        }
+
+        parentStack.append(ctx);
+    }
+
+    // Add the text content of the leaf
+    litehtml::tstring elemText;
+    leafElement->get_text(elemText);
+    if (!elemText.empty()) {
+        QString text = QString::fromStdString(elemText);
+        // HTML escape the text
+        text.replace(QLatin1Char('&'), QLatin1String("&amp;"));
+        text.replace(QLatin1Char('<'), QLatin1String("&lt;"));
+        text.replace(QLatin1Char('>'), QLatin1String("&gt;"));
+        html += text;
+    }
+}
+
+QString DocumentContainer::selectedHtml() const
+{
+    const Selection &sel = d->m_selection;
+    if (!sel.startElem.element || !sel.endElem.element) {
+        return QString();
+    }
+
+    // Get ordered start and end elements (same logic as Selection::update)
+    Selection::Element start;
+    Selection::Element end;
+    std::tie(start, end) = getStartAndEnd(sel.startElem, sel.endElem);
+
+    QString bodyHtml;
+    QVector<ElementContext> parentStack;
+
+    // Process start element
+    serializeLeafWithParents(start.element, parentStack, bodyHtml);
+
+    // If start and end are different, traverse all elements between them
+    if (start.element != end.element) {
+        litehtml::element::ptr current = start.element;
+        do {
+            current = nextLeaf(current, end.element);
+            if (current && current != end.element) {
+                serializeLeafWithParents(current, parentStack, bodyHtml);
+            }
+        } while (current && current != end.element);
+
+        // Process end element
+        if (end.element) {
+            serializeLeafWithParents(end.element, parentStack, bodyHtml);
+        }
+    }
+
+    // Close all remaining open tags
+    for (int i = parentStack.size() - 1; i >= 0; --i) {
+        if (parentStack[i].opened && !parentStack[i].tagName.isEmpty()) {
+            bodyHtml += QStringLiteral("</") + parentStack[i].tagName + QStringLiteral(">");
+        }
+    }
+
+    // Wrap in proper HTML document structure
+    QString html;
+    html += QStringLiteral("<!DOCTYPE html><html><head><meta charset=\"UTF-8\"></head><body>");
+    html += bodyHtml;
+    html += QStringLiteral("</body></html>");
+
+    return html;
+}
+
 void DocumentContainer::findText(const QString &text,
                                  QTextDocument::FindFlags flags,
                                  bool incremental,
@@ -1364,11 +1691,6 @@ void DocumentContainer::setLinkCallback(const DocumentContainer::LinkCallback &c
     d->m_linkCallback = callback;
 }
 
-void DocumentContainer::setCheckboxCallback(const DocumentContainer::CheckboxCallback &callback)
-{
-    d->m_checkboxCallback = callback;
-}
-
 void DocumentContainer::setPaletteCallback(const DocumentContainer::PaletteCallback &callback)
 {
     d->m_paletteCallback = callback;
