@@ -30,7 +30,10 @@
 
 #include "container_qpainter.h"
 
+#include <QClipboard>
 #include <QDebug>
+#include <QGuiApplication>
+#include <QMimeData>
 #include <QPaintEvent>
 #include <QPainter>
 #include <QScrollBar>
@@ -398,13 +401,6 @@ QLiteHtmlWidget::QLiteHtmlWidget(QWidget *parent)
         QMetaObject::invokeMethod(
             this, [this, fullUrl] { emit linkClicked(fullUrl); }, Qt::QueuedConnection);
     });
-    d->documentContainer.setCheckboxCallback([this](int index) {
-        // Emit a checkbox:// URL similar to the regular preview
-        QUrl checkboxUrl(QStringLiteral("checkbox://_%1").arg(index));
-        // delay because document may not be changed directly during this callback
-        QMetaObject::invokeMethod(
-            this, [this, checkboxUrl] { emit linkClicked(checkboxUrl); }, Qt::QueuedConnection);
-    });
     d->documentContainer.setClipboardCallback([this](bool yes) { emit copyAvailable(yes); });
 
     // TODO adapt mastercss to palette (default text & background color)
@@ -479,7 +475,7 @@ bool QLiteHtmlWidget::findText(const QString &text,
         .findText(text, flags, incremental, wrapped, &success, &oldSelection, &newSelection);
     // scroll to search result position and/or redraw as necessary
     QRect newSelectionCombined;
-    for (const QRect &r : qAsConst(newSelection))
+    for (const QRect &r : std::as_const(newSelection))
         newSelectionCombined = newSelectionCombined.united(r);
     QScrollBar *vBar = verticalScrollBar();
     const int top = newSelectionCombined.top();
@@ -490,7 +486,7 @@ bool QLiteHtmlWidget::findText(const QString &text,
         vBar->setValue(bottom);
     } else {
         viewport()->update(fromVirtual(newSelectionCombined.translated(-scrollPosition())));
-        for (const QRect &r : qAsConst(oldSelection))
+        for (const QRect &r : std::as_const(oldSelection))
             viewport()->update(fromVirtual(r.translated(-scrollPosition())));
     }
     return success;
@@ -533,6 +529,11 @@ QString QLiteHtmlWidget::selectedText() const
     return d->documentContainer.selectedText();
 }
 
+QString QLiteHtmlWidget::selectedHtml() const
+{
+    return d->documentContainer.selectedHtml();
+}
+
 void QLiteHtmlWidget::paintEvent(QPaintEvent *event)
 {
     if (!d->documentContainer.hasDocument())
@@ -572,7 +573,8 @@ void QLiteHtmlWidget::mousePressEvent(QMouseEvent *event)
     htmlPos(event->pos(), &viewportPos, &pos);
     const QVector<QRect> areas = d->documentContainer.mousePressEvent(pos,
                                                                       viewportPos,
-                                                                      event->button());
+                                                                      event->button(),
+                                                                      event->modifiers());
     for (const QRect &r : areas)
         viewport()->update(fromVirtual(r.translated(-scrollPosition())));
 }
@@ -640,6 +642,21 @@ void QLiteHtmlWidget::keyPressEvent(QKeyEvent *event)
             verticalScrollBar()->triggerAction(sliderAction);
             event->accept();
         }
+    } else if (event->modifiers() == Qt::ControlModifier && event->key() == Qt::Key_C) {
+        // Copy selected text to clipboard when Ctrl+C is pressed
+        const QString text = selectedText();
+        if (!text.isEmpty()) {
+            auto *mimeData = new QMimeData();
+            mimeData->setText(text);
+            // Also copy HTML if available
+            const QString html = selectedHtml();
+
+            if (!html.isEmpty()) {
+                mimeData->setHtml(html);
+            }
+            QGuiApplication::clipboard()->setMimeData(mimeData);
+        }
+        event->accept();
     }
 
     QAbstractScrollArea::keyPressEvent(event);
