@@ -1,7 +1,7 @@
 /*
  * MIT License
  *
- * Copyright (c) 2014-2025 Patrizio Bekerle -- <patrizio@bekerle.com>
+ * Copyright (c) 2014-2026 Patrizio Bekerle -- <patrizio@bekerle.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -126,6 +126,10 @@ void QMarkdownTextEdit::setLineNumbersOtherLineColor(QColor color) {
     _lineNumArea->setOtherLineColor(std::move(color));
 }
 
+void QMarkdownTextEdit::setBookmarkLines(const QHash<int, int> &bookmarkLines) {
+    _lineNumArea->setBookmarkLines(bookmarkLines);
+}
+
 void QMarkdownTextEdit::setSearchWidgetDebounceDelay(uint debounceDelay) {
     _debounceDelay = debounceDelay;
     searchWidget()->setDebounceDelay(_debounceDelay);
@@ -464,14 +468,26 @@ bool QMarkdownTextEdit::eventFilter(QObject *obj, QEvent *event) {
         _mouseButtonDown = false;
         auto *mouseEvent = static_cast<QMouseEvent *>(event);
 
-        // track `Ctrl + Click` in the text edit
+        // track `Ctrl + Click` and `Ctrl + Shift + Click` in the text edit
         if ((obj == this->viewport()) &&
-            (mouseEvent->button() == Qt::LeftButton) &&
-            (QGuiApplication::keyboardModifiers() == Qt::ExtraButton24)) {
-            // open the link (if any) at the current position
-            // in the noteTextEdit
-            openLinkAtCursorPosition();
-            return true;
+            (mouseEvent->button() == Qt::LeftButton)) {
+            auto modifiers = QGuiApplication::keyboardModifiers();
+
+            // Check for Ctrl+Shift+Click (open in new tab)
+            if (modifiers.testFlag(Qt::ControlModifier) &&
+                modifiers.testFlag(Qt::ShiftModifier)) {
+                _openLinkInNewTab = true;
+                openLinkAtCursorPosition();
+                _openLinkInNewTab = false;
+                return true;
+            }
+            // Check for Ctrl+Click (open in current tab)
+            else if (modifiers == Qt::ControlModifier ||
+                     modifiers == Qt::ExtraButton24) {
+                _openLinkInNewTab = false;
+                openLinkAtCursorPosition();
+                return true;
+            }
         }
     } else if (event->type() == QEvent::MouseButtonPress) {
         _mouseButtonDown = true;
@@ -1227,24 +1243,47 @@ bool QMarkdownTextEdit::openLinkAtCursorPosition() {
 
     qDebug() << __func__ << " - 'emit urlClicked( urlString )': " << urlString;
 
+    // Check if URL matches any ignored regular expressions
+    for (const QRegularExpression &regex : _ignoredClickUrlRegexps) {
+        if (regex.match(urlString).hasMatch()) {
+            return true;    // URL matches ignored regex, don't open it
+        }
+    }
+
+    // Return false if no URL was found at the cursor position
+    if (urlString.isEmpty()) {
+        return false;
+    }
+
+    // Emit signal to let derived classes handle URL opening
     Q_EMIT urlClicked(urlString);
 
+    // Check if there are signal handlers connected
+    const int numReceivers = receivers(SIGNAL(urlClicked(QString)));
+    qDebug() << __func__
+             << " - number of urlClicked receivers:" << numReceivers;
+    qDebug() << __func__ << " - _openLinkInNewTab:" << _openLinkInNewTab;
+
+    // If signal handlers are connected, they will handle opening the URL
+    // Don't call openUrl() here to avoid duplicate opening
+    if (numReceivers > 0) {
+        qDebug()
+            << __func__
+            << " - Signal handlers present, letting them handle URL opening";
+        return true;
+    }
+
+    // No signal handlers connected, use base class logic (backward
+    // compatibility)
+    qDebug() << __func__ << " - No signal handlers, using base class openUrl";
     if ((url.isValid() && isValidUrl(urlString)) || isRelativeFileUrl ||
         isLegacyAttachmentUrl) {
-        // Check if URL matches any ignored regular expressions
-        for (const QRegularExpression &regex : _ignoredClickUrlRegexps) {
-            if (regex.match(urlString).hasMatch()) {
-                return true; // URL matches ignored regex, don't open it
-            }
-        }
-
         // ignore some schemata
         if (!(_ignoredClickUrlSchemata.contains(url.scheme()) ||
               isRelativeFileUrl || isLegacyAttachmentUrl)) {
             // open the url
-            openUrl(urlString);
+            openUrl(urlString, _openLinkInNewTab);
         }
-
         return true;
     }
 
@@ -1272,7 +1311,8 @@ bool QMarkdownTextEdit::isValidUrl(const QString &urlString) {
  *   "/path/to/my/file/QOwnNotes.pdf" if the operating system supports that
  *  handler
  */
-void QMarkdownTextEdit::openUrl(const QString &urlString) {
+void QMarkdownTextEdit::openUrl(const QString &urlString, bool openInNewTab) {
+    Q_UNUSED(openInNewTab)
     qDebug() << "QMarkdownTextEdit " << __func__
              << " - 'urlString': " << urlString;
 
